;(function ( $, window, undefined ) {

    'use strict';
    var pluginName = 'gallery_medialist',
        document = window.document,
        defaults = {};

    function Plugin( element, options ) {
        
		this.element = element;
        this.options = $.extend( {}, defaults, options) ;
        
        this._defaults = defaults;
        this._name = pluginName;
		
		this.dragging = 0;
		this.typecount = {
			'thumbnail': 0,
			'icon': 0
		};

		this.get_filenames = function () {
			var str_filenames = $.trim($(element).find('input:eq(0)').val()),
				arr_filenames = [];
			if (str_filenames.length) {
				arr_filenames = str_filenames.split(',');
			}
			return arr_filenames;
		};
		
		this.get_icon = function (filename) {
			var ext = filename.substr(filename.lastIndexOf('.') + 1).toLowerCase(),
				extensionmap = {
					'jpg': filename,
					'jpeg': filename,
					'bmp': filename,
					'gif': filename,
					'png': filename,
					'doc': 1,
					'docx': 1,
					'eps': 1,
					'exe': 1,
					'flv': 1,
					'gz': 1,
					'java': 1,
					'mov': 1,
					'mp3': 1,
					'ogg': 1,
					'pdf': 1,
					'ppt': 1,
					'pptx': 1,
					'rar': 1,
					'rtf': 1,
					'swf': 1,
					'tar': 1,
					'tif': 1,
					'tiff': 1,
					'txt': 1,
					'wma': 1,
					'xls': 1,
					'xlsx': 1,
					'zip': 1
				};

			if (extensionmap[ext]) {
				if (extensionmap[ext] === filename) {
					this.typecount.thumbnail += 1;
					return ('/redaxo/index.php?rex_img_type=rex_mediapool_preview&rex_img_file=' + filename);
				}
				this.typecount.icon += 1;
				return 'media/mime-' + ext + '.gif';
			}
			this.typecount.icon += 1;
			return 'media/mime-unknown.gif';
		};
		
		this.get_filedata = function (filenames) {
			var that = this;
			$.ajax({
				'type': 'GET',
				'url': '/redaxo/index.php',
				'dataType': 'json',
				'data': {
					'page': 'addon',
					'addonname': 'be_style',
					'pluginname': 'redaxo-gallery-medialist',
					'subpage': 'help',
					'filenames': filenames.join(',')
				},
				'success': function (response) {
					that.update_filedata(response);
				}
			});
		};
		
		this.update_filedata = function (data) {
			$(this.element).find('ul li').each(function () {
				var $tmp = $('<div></div>'),
					current = data[$(this).attr('title')];
				$tmp.append('<span class="filename">' + current.filename + '</span>');
				$tmp.append('<span class="title">' + current.title + '</span>');
				$(this).find('div:eq(1)').html($tmp.html());
			});
		};
		
		this.set_filenames = function (current) {
			var arr_filenames = [],
				$select = $('<select></select>');
			
			current = (undefined !== current) ? current : -1;
			$(element).find('ul').find('li').each(function () {
				arr_filenames.push($(this).attr('title'));
			});
			$(element).find('input:eq(0)').val(arr_filenames.join(','));
			$.each(arr_filenames, function (idx) {
				if (current !== -1 && current === idx) {
					$select.append('<option selected="selected" value="' + this + '">' + this + '</option>');
				} else {
					$select.append('<option value="' + this + '">' + this + '</option>');
				}
			});
			$(this.element).find('select').html($select.html());
		};
		
		this.render_thumbnails = function (filenames) {
			var that = this,
				$list = $('<ul></ul>'),
				$select = $('<select></select>');
			$.each(filenames, function () {
				$list.append('<li title="' +  this + '"><div class="preview" style="background-image: url(' + that.get_icon(this) + ')"></div><div class="meta"></div></li>');
				$select.append('<option value="' + this + '">' + this + '</option>');
			});
			$(this.element).find('select').html($select.html());
			$list.delegate('li', 'click', function () {
				if ($(this).hasClass('chosen')) {
					$(this).removeClass('chosen');
					that.set_filenames();
				} else {
					$(this).siblings().removeClass('chosen').end().addClass('chosen');
					that.set_filenames($(this).prevAll().length);
				}
			});
			$list.sortable({
				'start': function(event, ui) {
					that.dragging = 1;
				},
				'stop': function(event, ui) {
					var current;
					$list.find('.chosen').each(function () {
						current = $(this).prevAll().length;
					});
					that.set_filenames(current);
					that.dragging = 0;
				},
				'tolerance': 'pointer'
			});
			return $list;
		};
		
		this.get_dragging_status = function () {
			return this.dragging;
		};
		
		this.render_viewcontrols = function () {
			var $el = $(this.element),
				$controls = $('<span class="viewcontrols"><span class="onlythumbs"><a class="onlythumbs">Miniaturen</a> | <a class="details">Details</a></span></span>');
			
			$controls.delegate('a', 'click', function () {
				var cur_class = $(this).attr('class');
				if ($(this).parent().hasClass(cur_class)) {
					return false;
				} else {
					$el.toggleClass('details');
					$(this).parent().attr('class', cur_class);
				}
			});
			
			return $controls;
		};
		
        this.init();
    }

    Plugin.prototype.init = function () {
		var that = this,
			$el = $(this.element),
			filenames = this.get_filenames();
		$el.parent().addClass('gallery-medialist');
		$el.find('.rex-media-preview').append(this.render_thumbnails(filenames));
		$el.find('.rex-widget-2col').append(this.render_viewcontrols());
		if (filenames.length) {
			this.get_filedata(filenames);
			if (this.typecount.icon > this.typecount.thumbnail) {
				$el.find('.onlythumbs:eq(0)').attr('class','details');
				$el.addClass('details');
			}
		}
		setInterval(function () {
			filenames = that.get_filenames();
			if (!that.dragging && filenames.length !== $el.find('ul').find('li').length) {
				$el.find('.rex-media-preview').empty().append(that.render_thumbnails(filenames));
				that.set_filenames();
				filenames = that.get_filenames();
				if (filenames.length) {
					that.get_filedata(filenames);
				}
			}
		}, 100);
    };

    $.fn[pluginName] = function ( options ) {
        return this.each(function () {
			if (!$(this).parents('.no-pimp').length) {
				if (!$.data(this, 'plugin_' + pluginName)) {
					$.data(this, 'plugin_' + pluginName, new Plugin( this, options ));
				}
            }
        });
    };

}(jQuery, window));